! WHIZARD 1.20 Feb 11 2002
! 
! (C) 1999-2001 by Wolfgang Kilian <kilian@particle.uni-karlsruhe.de>
!
! WHIZARD is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! WHIZARD is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This file has been stripped of most comments.  For documentation, refer
! to the source 'whizard.nw'

module pdflib_interface

  use kinds, only: default !NODEP!
  use kinds, only: double !NODEP!
  use diagnostics, only: msg_fatal

  implicit none
  private

  public :: pdflib_init, pdflib_strfun, pdflib_strfun_array
 
  integer, parameter :: TQUARK = 6, GLUON1 = 9, GLUON2 = 21
  integer, parameter :: PROTON = 2212, ANTIPROTON = -PROTON, PHOTON = 22

  integer, parameter, public :: PDFSET_ARRAY_SIZE = 20
  integer, parameter, public :: PDFSET_CHAR_LEN = 20

  logical, save :: pdf_invert = .false.

contains

  subroutine pdflib_init &
       & (pdg_code_in, ngroup, nset, nfl, lo, QCDL4, top_mass, first)
    integer, intent(in) :: pdg_code_in, ngroup, nset, nfl, lo
    real(kind=default), intent(in) :: QCDL4, top_mass
    logical, intent(in), optional :: first
    integer :: nptype, i
    character (len=PDFSET_CHAR_LEN), dimension(PDFSET_ARRAY_SIZE) :: parm
    real(kind=double), dimension(PDFSET_ARRAY_SIZE) :: value
    real(kind=double), parameter :: x_dummy = 0.5_double
    real(kind=double), parameter :: scale_dummy = 100._double
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf_dummy
    external pdfset, pftopdg
    select case (pdg_code_in)
    case (PROTON)
       nptype = 1
       pdf_invert = .false.
    case (ANTIPROTON)
       nptype = 1
       pdf_invert = .true.
    case (PHOTON)
       nptype = 3
       pdf_invert = .false.
    end select
    parm = " "
    value = 0
    i = 1
    if (nptype > 0) then
       parm (i) = "NPTYPE";  value(i) = nptype;   i = i+1
    end if
    if (ngroup > 0) then
       parm (i) = "NGROUP";  value(i) = ngroup;   i = i+1
    end if
    if (nset > 0) then
       parm (i) = "NSET";    value(i) = nset;     i = i+1
    end if
    if (nfl > 0) then
       parm (i) = "NFL";     value(i) = nfl;      i = i+1
    end if
    if (lo > 0) then
       parm (i) = "LO";      value(i) = lo;       i = i+1
    end if
    if (QCDL4 > 0) then
       parm (i) = "QCDL4";   value(i) = QCDL4;    i = i+1
    end if
    if (top_mass > 0) then
       parm (i) = "TMAS";    value(i) = top_mass; i = i+1
    end if
    call pdfset (parm, value)
    if (present(first)) then
       if (first)  call pftopdg (x_dummy, scale_dummy, dxpdf_dummy)
    end if
  end subroutine pdflib_init

  function pdflib_strfun (x, scale, pdg_code) result (f)
    real(kind=default), intent(in) :: x, scale
    integer, intent(in) :: pdg_code
    real(kind=default) :: f
    integer :: code
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf
    external pftopdg
    select case (pdg_code)
    case (GLUON1,GLUON2)
       code = 0
    case (-TQUARK:TQUARK)
       code = pdg_code
    case default
       call msg_fatal(" Structure function call with illegal particle code")
    end select
    if (pdf_invert)  code = -code
    call pftopdg (real(x,kind=double), real(scale,kind=double), dxpdf)
    f = real(dxpdf(code), kind=default) / x
  end function pdflib_strfun

  subroutine pdflib_strfun_array (x, scale, rho)
    real(kind=default), intent(in) :: x, scale
    real(kind=default), dimension(-TQUARK:TQUARK), intent(out) :: rho
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf
    external pftopdg
    call pftopdg (real(x,kind=double), real(scale,kind=double), dxpdf)
    if (.not.pdf_invert) then
       rho = dxpdf / x
    else
       rho = dxpdf(TQUARK:-TQUARK:-1) / x
    end if
  end subroutine pdflib_strfun_array

end module pdflib_interface
